import { address, getProgramDerivedAddress, getUtf8Encoder, getAddressEncoder, fixEncoderSize, getBytesEncoder, transformEncoder, getStructEncoder, getU8Encoder, getStructDecoder, fixDecoderSize, getBytesDecoder, getU8Decoder, getAddressDecoder, combineCodec, decodeAccount, assertAccountExists, fetchEncodedAccount, assertAccountsExist, fetchEncodedAccounts, createEncoder, createDecoder, getU16Encoder, getU16Decoder, getArrayEncoder, getArrayDecoder, getU64Encoder, getU64Decoder, getEnumEncoder, getEnumDecoder, getDiscriminatedUnionEncoder, getTupleEncoder, getDiscriminatedUnionDecoder, getTupleDecoder, getBooleanEncoder, getBooleanDecoder, getU32Encoder, getU32Decoder, addEncoderSizePrefix, addDecoderSizePrefix, getUtf8Decoder, getI64Encoder, getI32Encoder, getI64Decoder, getI32Decoder, containsBytes, getOptionEncoder, none, getOptionDecoder, AccountRole, isTransactionSigner as isTransactionSigner$1, upgradeRoleToSigner } from '@solana/web3.js';
import { findFeeVaultPda, resolveBuyerAta, resolvePoolAta, resolveMetadata, resolvePoolNftReceipt, resolveEditionFromTokenStandard, resolvePoolTokenRecordFromTokenStandard, resolveBuyerTokenRecordFromTokenStandard, TokenStandard, resolveTokenMetadataProgramFromTokenStandard, resolveSysvarInstructionsFromTokenStandard, resolveAuthorizationRulesProgramFromTokenStandard, resolveOwnerAta, resolveOwnerTokenRecordFromTokenStandard, resolveSellerAta, resolveSellerTokenRecordFromTokenStandard } from '@tensor-foundation/resolvers';

// src/generated/accounts/nftDepositReceipt.ts
async function findNftDepositReceiptPda(seeds, config = {}) {
  const {
    programAddress = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [
      getUtf8Encoder().encode("nft_receipt"),
      getAddressEncoder().encode(seeds.mint),
      getAddressEncoder().encode(seeds.pool)
    ]
  });
}
async function findPoolPda(seeds, config = {}) {
  const {
    programAddress = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [
      getUtf8Encoder().encode("pool"),
      getAddressEncoder().encode(seeds.owner),
      fixEncoderSize(getBytesEncoder(), 32).encode(seeds.poolId)
    ]
  });
}

// src/generated/accounts/nftDepositReceipt.ts
function getNftDepositReceiptEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["bump", getU8Encoder()],
      ["mint", getAddressEncoder()],
      ["pool", getAddressEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([206, 255, 132, 254, 67, 78, 62, 96])
    })
  );
}
function getNftDepositReceiptDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["bump", getU8Decoder()],
    ["mint", getAddressDecoder()],
    ["pool", getAddressDecoder()]
  ]);
}
function getNftDepositReceiptCodec() {
  return combineCodec(
    getNftDepositReceiptEncoder(),
    getNftDepositReceiptDecoder()
  );
}
function decodeNftDepositReceipt(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getNftDepositReceiptDecoder()
  );
}
async function fetchNftDepositReceipt(rpc, address3, config) {
  const maybeAccount = await fetchMaybeNftDepositReceipt(rpc, address3, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeNftDepositReceipt(rpc, address3, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address3, config);
  return decodeNftDepositReceipt(maybeAccount);
}
async function fetchAllNftDepositReceipt(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeNftDepositReceipt(
    rpc,
    addresses,
    config
  );
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeNftDepositReceipt(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map(
    (maybeAccount) => decodeNftDepositReceipt(maybeAccount)
  );
}
function getNftDepositReceiptSize() {
  return 73;
}
async function fetchNftDepositReceiptFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeNftDepositReceiptFromSeeds(
    rpc,
    seeds,
    config
  );
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeNftDepositReceiptFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address3] = await findNftDepositReceiptPda(seeds, { programAddress });
  return await fetchMaybeNftDepositReceipt(rpc, address3, fetchConfig);
}
var SOL_ADDRESS = address("11111111111111111111111111111111");
var isSol = (currency) => currency === SOL_ADDRESS;
var solCurrency = () => SOL_ADDRESS;
var getCurrencyEncoder = () => getAddressEncoder();
var getCurrencyDecoder = () => getAddressDecoder();
var getCurrencyCodec = () => combineCodec(getCurrencyEncoder(), getCurrencyDecoder());
var DEFAULT_ADDRESS = address("11111111111111111111111111111111");
var getNullableAddressEncoder = () => createEncoder({
  fixedSize: 32,
  write(value, bytes, offset) {
    if (value === null) {
      bytes.set(getAddressEncoder().encode(DEFAULT_ADDRESS), offset);
    } else {
      bytes.set(getAddressEncoder().encode(value), offset);
    }
    return offset + 32;
  }
});
var getNullableAddressDecoder = () => createDecoder({
  fixedSize: 32,
  read(bytes, offset) {
    if (getAddressDecoder().decode(bytes, offset) === DEFAULT_ADDRESS) {
      return [null, offset + 32];
    } else {
      return [getAddressDecoder().decode(bytes, offset), offset + 32];
    }
  }
});
var getNullableAddressCodec = () => combineCodec(getNullableAddressEncoder(), getNullableAddressDecoder());

// src/hooked/helpers.ts
var BASIS_POINTS = 10000n;
var BASIS_POINTS_DECIMAL_OFFSET = 4;
var BID_AMOUNT_LIMIT = 1e3;
function getNeededBalanceForBidQuantity({
  pool,
  bidQuantity
}) {
  if (bidQuantity < 1) return 0;
  if (pool.config.poolType === 1 /* NFT */) return 0;
  const excludeMMFee = pool.config.poolType === 2 /* Trade */ && !pool.config.mmCompoundFees;
  if (pool.config.curveType === 0 /* Linear */) {
    const currentPrice = calculatePrice({
      pool,
      side: 1 /* Sell */,
      royaltyFeeBps: 0,
      extraOffset: 0,
      excludeMMFee
    });
    if (bidQuantity === 1) return currentPrice;
    const maxPossibleBidsBeforeZero = 1 + Number(BigInt(currentPrice) / pool.config.delta);
    bidQuantity = Math.min(bidQuantity, maxPossibleBidsBeforeZero);
    const bases = currentPrice * bidQuantity;
    const deltas = Number(pool.config.delta) * (bidQuantity * (bidQuantity - 1)) / 2;
    return bases - deltas;
  } else {
    let totalPrice = 0;
    let i = 0;
    while (bidQuantity > 0) {
      totalPrice += calculatePrice({
        pool,
        side: 1 /* Sell */,
        royaltyFeeBps: 0,
        extraOffset: i,
        excludeMMFee
      });
      bidQuantity -= 1;
      if (totalPrice === 0) break;
      i += 1;
    }
    return totalPrice;
  }
}
function getAmountOfBids({
  pool,
  availableLamports
}) {
  if (pool.config.poolType === 1 /* NFT */) return 0;
  let amountOfBidsWithoutMaxCount;
  const excludeMMFee = pool.config.poolType === 2 /* Trade */ && !pool.config.mmCompoundFees;
  if (pool.config.curveType === 0 /* Linear */) {
    const currentPrice = calculatePrice({
      pool,
      side: 1 /* Sell */,
      royaltyFeeBps: 0,
      extraOffset: 0,
      excludeMMFee
    });
    if (currentPrice < 0) return 0;
    let maxPossibleBidsBeforeZero = 1 + Number(BigInt(currentPrice) / pool.config.delta);
    maxPossibleBidsBeforeZero = Math.min(
      maxPossibleBidsBeforeZero,
      BID_AMOUNT_LIMIT
    );
    let bidCount = 0;
    let accumulatedPrice = 0n;
    while (accumulatedPrice < BigInt(availableLamports) && bidCount < maxPossibleBidsBeforeZero + 1) {
      const price = calculatePrice({
        pool,
        side: 1 /* Sell */,
        royaltyFeeBps: 0,
        extraOffset: bidCount,
        excludeMMFee
      });
      accumulatedPrice += BigInt(price);
      bidCount += 1;
    }
    amountOfBidsWithoutMaxCount = Math.min(
      bidCount - 1,
      maxPossibleBidsBeforeZero
    );
  } else {
    let bidCount = 0;
    let accumulatedPrice = 0n;
    while (accumulatedPrice < BigInt(availableLamports) && bidCount < BID_AMOUNT_LIMIT + 1) {
      const price = calculatePrice({
        pool,
        side: 1 /* Sell */,
        royaltyFeeBps: 0,
        extraOffset: bidCount,
        excludeMMFee
      });
      accumulatedPrice += BigInt(price);
      bidCount += 1;
    }
    amountOfBidsWithoutMaxCount = bidCount - 1;
  }
  return isMaxTakerSellCountReached(pool) ? Math.min(
    amountOfBidsWithoutMaxCount,
    pool.maxTakerSellCount + pool.priceOffset
  ) : amountOfBidsWithoutMaxCount;
}
function getCurrentAskPrice({
  pool,
  royaltyFeeBps,
  extraOffset = 0,
  excludeMMFee = false
}) {
  if (pool.nftsHeld < 1) return null;
  if (isNotFulfillable({ pool, side: 0 /* Buy */ })) return null;
  return calculatePrice({
    pool,
    side: 0 /* Buy */,
    royaltyFeeBps,
    extraOffset,
    excludeMMFee
  });
}
function getCurrentBidPriceSync({
  pool,
  availableLamports,
  royaltyFeeBps,
  extraOffset = 0,
  excludeMMFee = false
}) {
  const bidPrice = calculatePrice({
    pool,
    side: 1 /* Sell */,
    royaltyFeeBps,
    extraOffset,
    excludeMMFee
  });
  if (bidPrice < 1) return 0;
  if (isNotFulfillable({ pool, side: 1 /* Sell */ })) return null;
  return availableLamports >= bidPrice ? bidPrice : null;
}
async function getCurrentBidPrice({
  rpc,
  pool,
  royaltyFeeBps,
  extraOffset = 0,
  excludeMMFee = false
}) {
  if (isNotFulfillable({ pool, side: 1 /* Sell */ })) return null;
  const bidPrice = calculatePrice({
    pool,
    side: 1 /* Sell */,
    royaltyFeeBps,
    extraOffset,
    excludeMMFee
  });
  if (bidPrice === null) return null;
  if (bidPrice < 1) return 0;
  if (!pool.sharedEscrow || pool.sharedEscrow === DEFAULT_ADDRESS)
    return pool.amount >= bidPrice ? bidPrice : null;
  const [escrowLamports, escrowDataLength] = await rpc.getAccountInfo(pool.sharedEscrow, { encoding: "base64" }).send().then((resp) => {
    const dataLength = resp.value?.data ? Buffer.from(resp.value.data[0], "base64").length : void 0;
    return [resp.value?.lamports, dataLength];
  });
  if (!escrowLamports || !escrowDataLength) return null;
  const rentExemption = await rpc.getMinimumBalanceForRentExemption(BigInt(escrowDataLength)).send();
  return BigInt(escrowLamports) - rentExemption >= bidPrice ? bidPrice : null;
}
function calculatePrice({
  pool,
  side,
  royaltyFeeBps,
  extraOffset = 0,
  excludeMMFee = false
}) {
  const extraOffsetNormalized = Math.abs(extraOffset);
  const tradePoolOffset = pool.config.poolType === 2 /* Trade */ && side === 1 /* Sell */ ? -1 : 0;
  const offset = pool.priceOffset + tradePoolOffset + (side === 1 /* Sell */ ? -extraOffsetNormalized : extraOffsetNormalized);
  const startingPrice = pool.config.startingPrice;
  const delta = pool.config.delta;
  let resultPrice;
  if (pool.config.curveType === 1 /* Exponential */) {
    const base = BASIS_POINTS + delta;
    const exponent = BigInt(Math.abs(offset));
    const [scaling, decimalOffset] = powerBpsAsBigInt_12DecimalMantissaPrecision(base, exponent);
    let resultPriceIntermediate;
    if (offset < 0) {
      const divident = startingPrice * 10n ** decimalOffset;
      const divisor = scaling;
      resultPriceIntermediate = divident / divisor + BigInt(+needsRoundingAddedBack(divident, divisor));
    } else {
      const divident = startingPrice * scaling;
      const divisor = 10n ** decimalOffset;
      resultPriceIntermediate = divident / divisor + BigInt(+needsRoundingAddedBack(divident, divisor));
    }
    resultPrice = Number(resultPriceIntermediate);
  } else {
    resultPrice = Number(startingPrice + delta * BigInt(offset));
  }
  let resultPriceIncludingMmFees;
  if (pool.config.poolType === 2 /* Trade */ && !excludeMMFee) {
    const mmFees = BigInt(resultPrice) * BigInt(pool.config.mmFeeBps ?? 0) / BASIS_POINTS;
    resultPriceIncludingMmFees = side === 1 /* Sell */ ? Number(BigInt(resultPrice) - mmFees) : Number(BigInt(resultPrice) + mmFees);
  } else {
    resultPriceIncludingMmFees = resultPrice;
  }
  let resultPriceIncludingMmFeesAndRoyalties;
  if (royaltyFeeBps === 0) return resultPriceIncludingMmFees;
  else {
    const royalties = BigInt(resultPrice) * BigInt(royaltyFeeBps) / BASIS_POINTS;
    resultPriceIncludingMmFeesAndRoyalties = side === 1 /* Sell */ ? Number(BigInt(resultPriceIncludingMmFees) - royalties) : Number(BigInt(resultPriceIncludingMmFees) + royalties);
  }
  return resultPriceIncludingMmFeesAndRoyalties;
}
function powerBpsAsBigInt_12DecimalMantissaPrecision(base, exponent) {
  let result;
  let decimalOffset = BASIS_POINTS_DECIMAL_OFFSET;
  let resultDecimalOffset = BASIS_POINTS_DECIMAL_OFFSET;
  if (exponent === 0n)
    return [1n * BASIS_POINTS, 1n * BigInt(BASIS_POINTS_DECIMAL_OFFSET)];
  if (exponent % 2n === 1n) {
    result = base;
  } else {
    result = 1n * BASIS_POINTS;
  }
  let pow = base;
  exponent /= 2n;
  while (exponent > 0n) {
    pow *= pow;
    decimalOffset *= 2;
    [pow, decimalOffset] = cutTo12MantissaDecimals(pow, decimalOffset);
    if (exponent % 2n === 1n) {
      result *= pow;
      resultDecimalOffset += decimalOffset;
      [result, resultDecimalOffset] = cutTo12MantissaDecimals(
        result,
        resultDecimalOffset
      );
    }
    exponent /= 2n;
  }
  [result, resultDecimalOffset] = cutTo12MantissaDecimals(
    result,
    resultDecimalOffset
  );
  return [result, BigInt(resultDecimalOffset)];
}
var needsRoundingAddedBack = (divident, divisor) => {
  const remainder = divident % divisor;
  return divisor - remainder <= remainder;
};
var cutTo12MantissaDecimals = (num, decimalOffset, withRounding = true) => {
  const decimalPrecision = 12;
  const exponentLength = num.toString(10).length - decimalOffset;
  let adjustedOffset = decimalOffset;
  let adjustedNum = num;
  if (decimalOffset > decimalPrecision) {
    adjustedNum = BigInt(
      num.toString(10).slice(0, exponentLength + decimalPrecision)
    );
    adjustedOffset = adjustedNum.toString(10).length - exponentLength;
  }
  adjustedNum += withRounding ? BigInt(+(parseInt(num.toString()[decimalPrecision + 1]) > 4)) : 0n;
  return [adjustedNum, adjustedOffset];
};
function isMaxTakerSellCountReached(pool) {
  return pool.priceOffset * -1 === pool.maxTakerSellCount && pool.maxTakerSellCount !== 0 && !!pool.sharedEscrow && pool.sharedEscrow !== DEFAULT_ADDRESS;
}
var isNotFulfillable = ({
  pool,
  side
}) => {
  return (
    // can't sell into PoolType.NFT
    pool.config.poolType === 1 /* NFT */ && side === 1 /* Sell */ || // can't buy from a PoolType.Token
    pool.config.poolType === 0 /* Token */ && side === 0 /* Buy */ || // if maxTakerSellCount is reached when pool is attached to margin acc,
    // pool can't fulfill more bids
    isMaxTakerSellCountReached(pool) && side === 1 /* Sell */
  );
};
var getNullableU16Encoder = () => createEncoder({
  fixedSize: 2,
  write(value, bytes, offset) {
    if (value === null) {
      bytes.set(getU16Encoder().encode(0), offset);
    } else {
      bytes.set(getU16Encoder().encode(value), offset);
    }
    return offset + 2;
  }
});
var getNullableU16Decoder = () => createDecoder({
  fixedSize: 2,
  read(bytes, offset) {
    if (getU16Decoder().decode(bytes, offset) === 0) {
      return [null, offset + 2];
    } else {
      return [getU16Decoder().decode(bytes, offset), offset + 2];
    }
  }
});
var getNullableU16Codec = () => combineCodec(getNullableU16Encoder(), getNullableU16Decoder());
function expectSome(value) {
  if (value == null) {
    throw new Error("Expected a value but received null or undefined.");
  }
  return value;
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function expectTransactionSigner(value) {
  if (!value || !isTransactionSigner(value)) {
    throw new Error("Expected a TransactionSigner.");
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? AccountRole.WRITABLE : AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && isTransactionSigner$1(value);
}

// src/hooked/resolvers.ts
var resolvePoolIdOnCreate = (_) => {
  return new Uint8Array(32).map(() => Math.random() * 256 & 255);
};
var resolveFeeVaultPdaFromPool = async ({
  accounts
}) => {
  return {
    value: await findFeeVaultPda({
      address: expectAddress(accounts.pool?.value)
    })
  };
};
function getAuthorizationDataLocalEncoder() {
  return getStructEncoder([
    ["payload", getArrayEncoder(getTaggedPayloadEncoder())]
  ]);
}
function getAuthorizationDataLocalDecoder() {
  return getStructDecoder([
    ["payload", getArrayDecoder(getTaggedPayloadDecoder())]
  ]);
}
function getAuthorizationDataLocalCodec() {
  return combineCodec(
    getAuthorizationDataLocalEncoder(),
    getAuthorizationDataLocalDecoder()
  );
}
function getBuySellEventEncoder() {
  return getStructEncoder([
    ["currentPrice", getU64Encoder()],
    ["takerFee", getU64Encoder()],
    ["mmFee", getU64Encoder()],
    ["creatorsFee", getU64Encoder()]
  ]);
}
function getBuySellEventDecoder() {
  return getStructDecoder([
    ["currentPrice", getU64Decoder()],
    ["takerFee", getU64Decoder()],
    ["mmFee", getU64Decoder()],
    ["creatorsFee", getU64Decoder()]
  ]);
}
function getBuySellEventCodec() {
  return combineCodec(getBuySellEventEncoder(), getBuySellEventDecoder());
}
var CurveType = /* @__PURE__ */ ((CurveType2) => {
  CurveType2[CurveType2["Linear"] = 0] = "Linear";
  CurveType2[CurveType2["Exponential"] = 1] = "Exponential";
  return CurveType2;
})(CurveType || {});
function getCurveTypeEncoder() {
  return getEnumEncoder(CurveType);
}
function getCurveTypeDecoder() {
  return getEnumDecoder(CurveType);
}
function getCurveTypeCodec() {
  return combineCodec(getCurveTypeEncoder(), getCurveTypeDecoder());
}
var Direction = /* @__PURE__ */ ((Direction2) => {
  Direction2[Direction2["Up"] = 0] = "Up";
  Direction2[Direction2["Down"] = 1] = "Down";
  return Direction2;
})(Direction || {});
function getDirectionEncoder() {
  return getEnumEncoder(Direction);
}
function getDirectionDecoder() {
  return getEnumDecoder(Direction);
}
function getDirectionCodec() {
  return combineCodec(getDirectionEncoder(), getDirectionDecoder());
}
function getPayloadTypeLocalEncoder() {
  return getDiscriminatedUnionEncoder([
    [
      "Pubkey",
      getStructEncoder([["fields", getTupleEncoder([getAddressEncoder()])]])
    ],
    [
      "Seeds",
      getStructEncoder([
        ["fields", getTupleEncoder([getSeedsVecLocalEncoder()])]
      ])
    ],
    [
      "MerkleProof",
      getStructEncoder([
        ["fields", getTupleEncoder([getProofInfoLocalEncoder()])]
      ])
    ],
    [
      "Number",
      getStructEncoder([["fields", getTupleEncoder([getU64Encoder()])]])
    ]
  ]);
}
function getPayloadTypeLocalDecoder() {
  return getDiscriminatedUnionDecoder([
    [
      "Pubkey",
      getStructDecoder([["fields", getTupleDecoder([getAddressDecoder()])]])
    ],
    [
      "Seeds",
      getStructDecoder([
        ["fields", getTupleDecoder([getSeedsVecLocalDecoder()])]
      ])
    ],
    [
      "MerkleProof",
      getStructDecoder([
        ["fields", getTupleDecoder([getProofInfoLocalDecoder()])]
      ])
    ],
    [
      "Number",
      getStructDecoder([["fields", getTupleDecoder([getU64Decoder()])]])
    ]
  ]);
}
function getPayloadTypeLocalCodec() {
  return combineCodec(
    getPayloadTypeLocalEncoder(),
    getPayloadTypeLocalDecoder()
  );
}
function payloadTypeLocal(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isPayloadTypeLocal(kind, value) {
  return value.__kind === kind;
}
function getPoolConfigEncoder() {
  return getStructEncoder([
    ["poolType", getPoolTypeEncoder()],
    ["curveType", getCurveTypeEncoder()],
    ["startingPrice", getU64Encoder()],
    ["delta", getU64Encoder()],
    ["mmCompoundFees", getBooleanEncoder()],
    ["mmFeeBps", getNullableU16Encoder()]
  ]);
}
function getPoolConfigDecoder() {
  return getStructDecoder([
    ["poolType", getPoolTypeDecoder()],
    ["curveType", getCurveTypeDecoder()],
    ["startingPrice", getU64Decoder()],
    ["delta", getU64Decoder()],
    ["mmCompoundFees", getBooleanDecoder()],
    ["mmFeeBps", getNullableU16Decoder()]
  ]);
}
function getPoolConfigCodec() {
  return combineCodec(getPoolConfigEncoder(), getPoolConfigDecoder());
}
function getPoolStatsEncoder() {
  return getStructEncoder([
    ["takerSellCount", getU32Encoder()],
    ["takerBuyCount", getU32Encoder()],
    ["accumulatedMmProfit", getU64Encoder()]
  ]);
}
function getPoolStatsDecoder() {
  return getStructDecoder([
    ["takerSellCount", getU32Decoder()],
    ["takerBuyCount", getU32Decoder()],
    ["accumulatedMmProfit", getU64Decoder()]
  ]);
}
function getPoolStatsCodec() {
  return combineCodec(getPoolStatsEncoder(), getPoolStatsDecoder());
}
var PoolType = /* @__PURE__ */ ((PoolType2) => {
  PoolType2[PoolType2["Token"] = 0] = "Token";
  PoolType2[PoolType2["NFT"] = 1] = "NFT";
  PoolType2[PoolType2["Trade"] = 2] = "Trade";
  return PoolType2;
})(PoolType || {});
function getPoolTypeEncoder() {
  return getEnumEncoder(PoolType);
}
function getPoolTypeDecoder() {
  return getEnumDecoder(PoolType);
}
function getPoolTypeCodec() {
  return combineCodec(getPoolTypeEncoder(), getPoolTypeDecoder());
}
function getProofInfoLocalEncoder() {
  return getStructEncoder([
    ["proof", getArrayEncoder(fixEncoderSize(getBytesEncoder(), 32))]
  ]);
}
function getProofInfoLocalDecoder() {
  return getStructDecoder([
    ["proof", getArrayDecoder(fixDecoderSize(getBytesDecoder(), 32))]
  ]);
}
function getProofInfoLocalCodec() {
  return combineCodec(getProofInfoLocalEncoder(), getProofInfoLocalDecoder());
}
function getSeedsVecLocalEncoder() {
  return getStructEncoder([
    [
      "seeds",
      getArrayEncoder(addEncoderSizePrefix(getBytesEncoder(), getU32Encoder()))
    ]
  ]);
}
function getSeedsVecLocalDecoder() {
  return getStructDecoder([
    [
      "seeds",
      getArrayDecoder(addDecoderSizePrefix(getBytesDecoder(), getU32Decoder()))
    ]
  ]);
}
function getSeedsVecLocalCodec() {
  return combineCodec(getSeedsVecLocalEncoder(), getSeedsVecLocalDecoder());
}
function getTaggedPayloadEncoder() {
  return getStructEncoder([
    ["name", addEncoderSizePrefix(getUtf8Encoder(), getU32Encoder())],
    ["payload", getPayloadTypeLocalEncoder()]
  ]);
}
function getTaggedPayloadDecoder() {
  return getStructDecoder([
    ["name", addDecoderSizePrefix(getUtf8Decoder(), getU32Decoder())],
    ["payload", getPayloadTypeLocalDecoder()]
  ]);
}
function getTaggedPayloadCodec() {
  return combineCodec(getTaggedPayloadEncoder(), getTaggedPayloadDecoder());
}
var TakerSide = /* @__PURE__ */ ((TakerSide2) => {
  TakerSide2[TakerSide2["Buy"] = 0] = "Buy";
  TakerSide2[TakerSide2["Sell"] = 1] = "Sell";
  return TakerSide2;
})(TakerSide || {});
function getTakerSideEncoder() {
  return getEnumEncoder(TakerSide);
}
function getTakerSideDecoder() {
  return getEnumDecoder(TakerSide);
}
function getTakerSideCodec() {
  return combineCodec(getTakerSideEncoder(), getTakerSideDecoder());
}
function getTAmmEventEncoder() {
  return getDiscriminatedUnionEncoder([
    [
      "BuySellEvent",
      getStructEncoder([
        ["fields", getTupleEncoder([getBuySellEventEncoder()])]
      ])
    ]
  ]);
}
function getTAmmEventDecoder() {
  return getDiscriminatedUnionDecoder([
    [
      "BuySellEvent",
      getStructDecoder([
        ["fields", getTupleDecoder([getBuySellEventDecoder()])]
      ])
    ]
  ]);
}
function getTAmmEventCodec() {
  return combineCodec(getTAmmEventEncoder(), getTAmmEventDecoder());
}
function tAmmEvent(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isTAmmEvent(kind, value) {
  return value.__kind === kind;
}

// src/generated/accounts/pool.ts
function getPoolEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["version", getU8Encoder()],
      ["bump", getArrayEncoder(getU8Encoder(), { size: 1 })],
      ["poolId", fixEncoderSize(getBytesEncoder(), 32)],
      ["createdAt", getI64Encoder()],
      ["updatedAt", getI64Encoder()],
      ["expiry", getI64Encoder()],
      ["owner", getAddressEncoder()],
      ["whitelist", getAddressEncoder()],
      ["rentPayer", getAddressEncoder()],
      ["currency", getCurrencyEncoder()],
      ["amount", getU64Encoder()],
      ["priceOffset", getI32Encoder()],
      ["nftsHeld", getU32Encoder()],
      ["stats", getPoolStatsEncoder()],
      ["sharedEscrow", getNullableAddressEncoder()],
      ["cosigner", getNullableAddressEncoder()],
      ["makerBroker", getNullableAddressEncoder()],
      ["maxTakerSellCount", getU32Encoder()],
      ["config", getPoolConfigEncoder()],
      ["reserved", getArrayEncoder(getU8Encoder(), { size: 100 })]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([241, 154, 109, 4, 17, 177, 109, 188])
    })
  );
}
function getPoolDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["version", getU8Decoder()],
    ["bump", getArrayDecoder(getU8Decoder(), { size: 1 })],
    ["poolId", fixDecoderSize(getBytesDecoder(), 32)],
    ["createdAt", getI64Decoder()],
    ["updatedAt", getI64Decoder()],
    ["expiry", getI64Decoder()],
    ["owner", getAddressDecoder()],
    ["whitelist", getAddressDecoder()],
    ["rentPayer", getAddressDecoder()],
    ["currency", getCurrencyDecoder()],
    ["amount", getU64Decoder()],
    ["priceOffset", getI32Decoder()],
    ["nftsHeld", getU32Decoder()],
    ["stats", getPoolStatsDecoder()],
    ["sharedEscrow", getNullableAddressDecoder()],
    ["cosigner", getNullableAddressDecoder()],
    ["makerBroker", getNullableAddressDecoder()],
    ["maxTakerSellCount", getU32Decoder()],
    ["config", getPoolConfigDecoder()],
    ["reserved", getArrayDecoder(getU8Decoder(), { size: 100 })]
  ]);
}
function getPoolCodec() {
  return combineCodec(getPoolEncoder(), getPoolDecoder());
}
function decodePool(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getPoolDecoder()
  );
}
async function fetchPool(rpc, address3, config) {
  const maybeAccount = await fetchMaybePool(rpc, address3, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybePool(rpc, address3, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address3, config);
  return decodePool(maybeAccount);
}
async function fetchAllPool(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybePool(rpc, addresses, config);
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybePool(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodePool(maybeAccount));
}
function getPoolSize() {
  return 447;
}
async function fetchPoolFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybePoolFromSeeds(rpc, seeds, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybePoolFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address3] = await findPoolPda(seeds, { programAddress });
  return await fetchMaybePool(rpc, address3, fetchConfig);
}

// src/generated/errors/tensorAmm.ts
var TENSOR_AMM_ERROR__INVALID_PROOF = 12e3;
var TENSOR_AMM_ERROR__WHITELIST_NOT_VERIFIED = 12001;
var TENSOR_AMM_ERROR__BAD_WHITELIST = 12002;
var TENSOR_AMM_ERROR__WRONG_POOL_TYPE = 12003;
var TENSOR_AMM_ERROR__BAD_FEE_ACCOUNT = 12004;
var TENSOR_AMM_ERROR__BAD_ESCROW_ACCOUNT = 12005;
var TENSOR_AMM_ERROR__MISSING_FEES = 12006;
var TENSOR_AMM_ERROR__FEES_TOO_HIGH = 12007;
var TENSOR_AMM_ERROR__DELTA_TOO_LARGE = 12008;
var TENSOR_AMM_ERROR__ARITHMETIC_ERROR = 12009;
var TENSOR_AMM_ERROR__WRONG_POOL = 12010;
var TENSOR_AMM_ERROR__ROYALTIES_ENABLED = 12011;
var TENSOR_AMM_ERROR__PRICE_MISMATCH = 12012;
var TENSOR_AMM_ERROR__EXISTING_NFTS = 12013;
var TENSOR_AMM_ERROR__WRONG_MINT = 12014;
var TENSOR_AMM_ERROR__INSUFFICIENT_TSWAP_ACC_BALANCE = 12015;
var TENSOR_AMM_ERROR__BAD_OWNER = 12016;
var TENSOR_AMM_ERROR__FEES_NOT_ALLOWED = 12017;
var TENSOR_AMM_ERROR__BAD_METADATA = 12018;
var TENSOR_AMM_ERROR__CREATOR_MISMATCH = 12019;
var TENSOR_AMM_ERROR__WRONG_POOL_VERSION = 12020;
var TENSOR_AMM_ERROR__POOLS_ARE_THE_SAME = 12021;
var TENSOR_AMM_ERROR__WRONG_AUTHORITY = 12022;
var TENSOR_AMM_ERROR__FROZEN_AMOUNT_MISMATCH = 12023;
var TENSOR_AMM_ERROR__BAD_MINT_PROOF = 12024;
var TENSOR_AMM_ERROR__BAD_COSIGNER = 12025;
var TENSOR_AMM_ERROR__POOL_FROZEN = 12026;
var TENSOR_AMM_ERROR__BAD_SHARED_ESCROW = 12027;
var TENSOR_AMM_ERROR__POOL_NOT_ON_SHARED_ESCROW = 12028;
var TENSOR_AMM_ERROR__POOL_ON_SHARED_ESCROW = 12029;
var TENSOR_AMM_ERROR__WRONG_ORDER_TYPE = 12030;
var TENSOR_AMM_ERROR__WRONG_FROZEN_STATUS = 12031;
var TENSOR_AMM_ERROR__SHARED_ESCROW_IN_USE = 12032;
var TENSOR_AMM_ERROR__MAX_TAKER_SELL_COUNT_EXCEEDED = 12033;
var TENSOR_AMM_ERROR__MAX_TAKER_SELL_COUNT_TOO_SMALL = 12034;
var TENSOR_AMM_ERROR__BAD_RULE_SET = 12035;
var TENSOR_AMM_ERROR__POOL_FEES_COMPOUNDED = 12036;
var TENSOR_AMM_ERROR__BAD_ROYALTIES_PCT = 12037;
var TENSOR_AMM_ERROR__STARTING_PRICE_TOO_SMALL = 12038;
var TENSOR_AMM_ERROR__POOL_KEEP_ALIVE = 12039;
var TENSOR_AMM_ERROR__WRONG_RENT_PAYER = 12040;
var TENSOR_AMM_ERROR__SPL_TOKENS_NOT_SUPPORTED = 12041;
var TENSOR_AMM_ERROR__EXPIRY_TOO_LARGE = 12042;
var TENSOR_AMM_ERROR__EXPIRED_POOL = 12043;
var TENSOR_AMM_ERROR__POOL_NOT_EXPIRED = 12044;
var TENSOR_AMM_ERROR__UNSUPPORTED_CURRENCY = 12045;
var TENSOR_AMM_ERROR__INVALID_POOL_AMOUNT = 12046;
var TENSOR_AMM_ERROR__WRONG_MAKER_BROKER = 12047;
var TENSOR_AMM_ERROR__WRONG_OWNER = 12048;
var TENSOR_AMM_ERROR__ESCROW_PROGRAM_NOT_SET = 12049;
var TENSOR_AMM_ERROR__MINT_PROOF_NOT_SET = 12050;
var TENSOR_AMM_ERROR__MISSING_MAKER_BROKER = 12051;
var TENSOR_AMM_ERROR__MISSING_COSIGNER = 12052;
var tensorAmmErrorMessages;
if (process.env.NODE_ENV !== "production") {
  tensorAmmErrorMessages = {
    [TENSOR_AMM_ERROR__ARITHMETIC_ERROR]: `arithmetic error`,
    [TENSOR_AMM_ERROR__BAD_COSIGNER]: `bad cosigner passed - either wrong key or no signature`,
    [TENSOR_AMM_ERROR__BAD_ESCROW_ACCOUNT]: `escrow account doesn't match that stored on pool`,
    [TENSOR_AMM_ERROR__BAD_FEE_ACCOUNT]: `fee account doesn't match that stored on pool`,
    [TENSOR_AMM_ERROR__BAD_METADATA]: `metadata account does not match`,
    [TENSOR_AMM_ERROR__BAD_MINT_PROOF]: `mint proof account does not match`,
    [TENSOR_AMM_ERROR__BAD_OWNER]: `bad owner`,
    [TENSOR_AMM_ERROR__BAD_ROYALTIES_PCT]: `royalties percentage passed in must be between 0 and 100`,
    [TENSOR_AMM_ERROR__BAD_RULE_SET]: `rule set for programmable nft does not match`,
    [TENSOR_AMM_ERROR__BAD_SHARED_ESCROW]: `bad shared escrow account passed`,
    [TENSOR_AMM_ERROR__BAD_WHITELIST]: `unexpected whitelist address`,
    [TENSOR_AMM_ERROR__CREATOR_MISMATCH]: `provided creator address does not match metadata creator`,
    [TENSOR_AMM_ERROR__DELTA_TOO_LARGE]: `delta too large`,
    [TENSOR_AMM_ERROR__ESCROW_PROGRAM_NOT_SET]: `Escrow program not set`,
    [TENSOR_AMM_ERROR__EXISTING_NFTS]: `cannot close pool with nfts in escrow -- withdraw all before closing`,
    [TENSOR_AMM_ERROR__EXPIRED_POOL]: `Expired Pool`,
    [TENSOR_AMM_ERROR__EXPIRY_TOO_LARGE]: `Expiry too large`,
    [TENSOR_AMM_ERROR__FEES_NOT_ALLOWED]: `fees not allowed for non-trade pools`,
    [TENSOR_AMM_ERROR__FEES_TOO_HIGH]: `fees entered above allowed threshold`,
    [TENSOR_AMM_ERROR__FROZEN_AMOUNT_MISMATCH]: `amount frozen doesnt match current price`,
    [TENSOR_AMM_ERROR__INSUFFICIENT_TSWAP_ACC_BALANCE]: `insufficient Tswap account balance`,
    [TENSOR_AMM_ERROR__INVALID_POOL_AMOUNT]: `Invalid pool amount`,
    [TENSOR_AMM_ERROR__INVALID_PROOF]: `invalid merkle proof, token not whitelisted`,
    [TENSOR_AMM_ERROR__MAX_TAKER_SELL_COUNT_EXCEEDED]: `max taker sell count exceeded, pool cannot buy anymore NFTs`,
    [TENSOR_AMM_ERROR__MAX_TAKER_SELL_COUNT_TOO_SMALL]: `max taker sell count is too small`,
    [TENSOR_AMM_ERROR__MINT_PROOF_NOT_SET]: `Mint proof not set`,
    [TENSOR_AMM_ERROR__MISSING_COSIGNER]: `Missing cosigner account`,
    [TENSOR_AMM_ERROR__MISSING_FEES]: `when setting up a Trade pool, must provide fee bps`,
    [TENSOR_AMM_ERROR__MISSING_MAKER_BROKER]: `Missing maker broker account`,
    [TENSOR_AMM_ERROR__POOL_FEES_COMPOUNDED]: `this pool compounds fees and they cannot be withdrawn separately`,
    [TENSOR_AMM_ERROR__POOL_FROZEN]: `pool is frozen and cannot execute normal operations`,
    [TENSOR_AMM_ERROR__POOL_KEEP_ALIVE]: `Pool must keep minimum rent balance`,
    [TENSOR_AMM_ERROR__POOL_NOT_EXPIRED]: `Pool not expired`,
    [TENSOR_AMM_ERROR__POOL_NOT_ON_SHARED_ESCROW]: `expected a shared escrow pool to be passed in`,
    [TENSOR_AMM_ERROR__POOL_ON_SHARED_ESCROW]: `expected a non-shared escrow pool to be passed in`,
    [TENSOR_AMM_ERROR__POOLS_ARE_THE_SAME]: `new pool should not match old pool`,
    [TENSOR_AMM_ERROR__PRICE_MISMATCH]: `specified price not within current price`,
    [TENSOR_AMM_ERROR__ROYALTIES_ENABLED]: `royalties are enabled always`,
    [TENSOR_AMM_ERROR__SHARED_ESCROW_IN_USE]: `shared escrow account has pools open and is in use`,
    [TENSOR_AMM_ERROR__SPL_TOKENS_NOT_SUPPORTED]: `SPL tokens not supported`,
    [TENSOR_AMM_ERROR__STARTING_PRICE_TOO_SMALL]: `starting price can't be smaller than 1 lamport`,
    [TENSOR_AMM_ERROR__UNSUPPORTED_CURRENCY]: `Unsupported currency`,
    [TENSOR_AMM_ERROR__WHITELIST_NOT_VERIFIED]: `whitelist not verified -- currently only verified pools supported`,
    [TENSOR_AMM_ERROR__WRONG_AUTHORITY]: `wrong nft authority account provided`,
    [TENSOR_AMM_ERROR__WRONG_FROZEN_STATUS]: `wrong frozen status`,
    [TENSOR_AMM_ERROR__WRONG_MAKER_BROKER]: `Wrong maker broker account`,
    [TENSOR_AMM_ERROR__WRONG_MINT]: `wrong mint passed for provided accounts`,
    [TENSOR_AMM_ERROR__WRONG_ORDER_TYPE]: `wrong order type`,
    [TENSOR_AMM_ERROR__WRONG_OWNER]: `Wrong rent payer`,
    [TENSOR_AMM_ERROR__WRONG_POOL]: `this nft doesnt belong to this pool`,
    [TENSOR_AMM_ERROR__WRONG_POOL_TYPE]: `operation not permitted on this pool type`,
    [TENSOR_AMM_ERROR__WRONG_POOL_VERSION]: `wrong pool version provided`,
    [TENSOR_AMM_ERROR__WRONG_RENT_PAYER]: `Wrong rent payer`
  };
}
function getTensorAmmErrorMessage(code) {
  if (process.env.NODE_ENV !== "production") {
    return tensorAmmErrorMessages[code];
  }
  return "Error message not available in production bundles.";
}
var TENSOR_AMM_PROGRAM_ADDRESS = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
var TensorAmmAccount = /* @__PURE__ */ ((TensorAmmAccount2) => {
  TensorAmmAccount2[TensorAmmAccount2["NftDepositReceipt"] = 0] = "NftDepositReceipt";
  TensorAmmAccount2[TensorAmmAccount2["Pool"] = 1] = "Pool";
  return TensorAmmAccount2;
})(TensorAmmAccount || {});
function identifyTensorAmmAccount(account) {
  const data = account instanceof Uint8Array ? account : account.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([206, 255, 132, 254, 67, 78, 62, 96])
    ),
    0
  )) {
    return 0 /* NftDepositReceipt */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([241, 154, 109, 4, 17, 177, 109, 188])
    ),
    0
  )) {
    return 1 /* Pool */;
  }
  throw new Error(
    "The provided account could not be identified as a tensorAmm account."
  );
}
var TensorAmmInstruction = /* @__PURE__ */ ((TensorAmmInstruction2) => {
  TensorAmmInstruction2[TensorAmmInstruction2["TammNoop"] = 0] = "TammNoop";
  TensorAmmInstruction2[TensorAmmInstruction2["CreatePool"] = 1] = "CreatePool";
  TensorAmmInstruction2[TensorAmmInstruction2["EditPool"] = 2] = "EditPool";
  TensorAmmInstruction2[TensorAmmInstruction2["ClosePool"] = 3] = "ClosePool";
  TensorAmmInstruction2[TensorAmmInstruction2["CloseExpiredPool"] = 4] = "CloseExpiredPool";
  TensorAmmInstruction2[TensorAmmInstruction2["DepositNft"] = 5] = "DepositNft";
  TensorAmmInstruction2[TensorAmmInstruction2["WithdrawNft"] = 6] = "WithdrawNft";
  TensorAmmInstruction2[TensorAmmInstruction2["DepositSol"] = 7] = "DepositSol";
  TensorAmmInstruction2[TensorAmmInstruction2["WithdrawSol"] = 8] = "WithdrawSol";
  TensorAmmInstruction2[TensorAmmInstruction2["BuyNft"] = 9] = "BuyNft";
  TensorAmmInstruction2[TensorAmmInstruction2["SellNftTokenPool"] = 10] = "SellNftTokenPool";
  TensorAmmInstruction2[TensorAmmInstruction2["SellNftTradePool"] = 11] = "SellNftTradePool";
  TensorAmmInstruction2[TensorAmmInstruction2["BuyNftT22"] = 12] = "BuyNftT22";
  TensorAmmInstruction2[TensorAmmInstruction2["DepositNftT22"] = 13] = "DepositNftT22";
  TensorAmmInstruction2[TensorAmmInstruction2["SellNftTokenPoolT22"] = 14] = "SellNftTokenPoolT22";
  TensorAmmInstruction2[TensorAmmInstruction2["SellNftTradePoolT22"] = 15] = "SellNftTradePoolT22";
  TensorAmmInstruction2[TensorAmmInstruction2["WithdrawNftT22"] = 16] = "WithdrawNftT22";
  return TensorAmmInstruction2;
})(TensorAmmInstruction || {});
function identifyTensorAmmInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([31, 162, 228, 158, 153, 160, 198, 182])
    ),
    0
  )) {
    return 0 /* TammNoop */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([233, 146, 209, 142, 207, 104, 64, 188])
    ),
    0
  )) {
    return 1 /* CreatePool */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([50, 174, 34, 36, 3, 166, 29, 204])
    ),
    0
  )) {
    return 2 /* EditPool */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([140, 189, 209, 23, 239, 62, 239, 11])
    ),
    0
  )) {
    return 3 /* ClosePool */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([108, 212, 233, 53, 132, 83, 63, 219])
    ),
    0
  )) {
    return 4 /* CloseExpiredPool */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([93, 226, 132, 166, 141, 9, 48, 101])
    ),
    0
  )) {
    return 5 /* DepositNft */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([142, 181, 191, 149, 82, 175, 216, 100])
    ),
    0
  )) {
    return 6 /* WithdrawNft */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([108, 81, 78, 117, 125, 155, 56, 200])
    ),
    0
  )) {
    return 7 /* DepositSol */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([145, 131, 74, 136, 65, 137, 42, 38])
    ),
    0
  )) {
    return 8 /* WithdrawSol */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([96, 0, 28, 190, 49, 107, 83, 222])
    ),
    0
  )) {
    return 9 /* BuyNft */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([57, 44, 192, 48, 83, 8, 107, 48])
    ),
    0
  )) {
    return 10 /* SellNftTokenPool */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([131, 82, 125, 77, 13, 157, 36, 90])
    ),
    0
  )) {
    return 11 /* SellNftTradePool */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([155, 219, 126, 245, 170, 199, 51, 79])
    ),
    0
  )) {
    return 12 /* BuyNftT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([208, 34, 6, 147, 95, 218, 49, 160])
    ),
    0
  )) {
    return 13 /* DepositNftT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([149, 234, 31, 103, 26, 36, 166, 49])
    ),
    0
  )) {
    return 14 /* SellNftTokenPoolT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([124, 145, 23, 52, 72, 113, 85, 9])
    ),
    0
  )) {
    return 15 /* SellNftTradePoolT22 */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([112, 55, 80, 231, 181, 190, 92, 12])
    ),
    0
  )) {
    return 16 /* WithdrawNftT22 */;
  }
  throw new Error(
    "The provided instruction could not be identified as a tensorAmm instruction."
  );
}

// src/generated/instructions/buyNft.ts
function getBuyNftInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([96, 0, 28, 190, 49, 107, 83, 222]),
      authorizationData: value.authorizationData ?? none(),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? none()
    })
  );
}
function getBuyNftInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())]
  ]);
}
function getBuyNftInstructionDataCodec() {
  return combineCodec(
    getBuyNftInstructionDataEncoder(),
    getBuyNftInstructionDataDecoder()
  );
}
async function getBuyNftInstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    buyer: { value: input.buyer ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    buyerTokenRecord: {
      value: input.buyerTokenRecord ?? null,
      isWritable: true
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromPool(resolverScope)
    };
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.buyerTa.value) {
    accounts.buyerTa = {
      ...accounts.buyerTa,
      ...await resolveBuyerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.poolTokenRecord.value) {
    accounts.poolTokenRecord = {
      ...accounts.poolTokenRecord,
      ...await resolvePoolTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.buyerTokenRecord.value) {
    accounts.buyerTokenRecord = {
      ...accounts.buyerTokenRecord,
      ...await resolveBuyerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.buyerTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyNftInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyNftInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    buyer: { value: input.buyer ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    buyerTokenRecord: {
      value: input.buyerTokenRecord ?? null,
      isWritable: true
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.buyerTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyNftInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyNftInstruction(instruction) {
  if (instruction.accounts.length < 25) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      buyer: getNextAccount(),
      rentPayer: getNextAccount(),
      feeVault: getNextAccount(),
      pool: getNextAccount(),
      buyerTa: getNextAccount(),
      poolTa: getNextAccount(),
      mint: getNextAccount(),
      metadata: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      edition: getNextAccount(),
      poolTokenRecord: getNextOptionalAccount(),
      buyerTokenRecord: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      sharedEscrow: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      takerBroker: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount(),
      ammProgram: getNextAccount()
    },
    data: getBuyNftInstructionDataDecoder().decode(instruction.data)
  };
}
function getBuyNftT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["maxAmount", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([155, 219, 126, 245, 170, 199, 51, 79])
    })
  );
}
function getBuyNftT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["maxAmount", getU64Decoder()]
  ]);
}
function getBuyNftT22InstructionDataCodec() {
  return combineCodec(
    getBuyNftT22InstructionDataEncoder(),
    getBuyNftT22InstructionDataDecoder()
  );
}
async function getBuyNftT22InstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    buyer: { value: input.buyer ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromPool(resolverScope)
    };
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.buyerTa.value) {
    accounts.buyerTa = {
      ...accounts.buyerTa,
      ...await resolveBuyerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...args.transferHookAccounts.map((address3) => ({
      address: address3,
      role: AccountRole.READONLY
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyNftT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getBuyNftT22Instruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    buyer: { value: input.buyer ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    buyerTa: { value: input.buyerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...args.transferHookAccounts.map((address3) => ({
      address: address3,
      role: AccountRole.READONLY
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.buyer),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.buyerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getBuyNftT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBuyNftT22Instruction(instruction) {
  if (instruction.accounts.length < 17) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      buyer: getNextAccount(),
      rentPayer: getNextAccount(),
      feeVault: getNextAccount(),
      pool: getNextAccount(),
      buyerTa: getNextAccount(),
      poolTa: getNextAccount(),
      mint: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      sharedEscrow: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      takerBroker: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount(),
      ammProgram: getNextAccount()
    },
    data: getBuyNftT22InstructionDataDecoder().decode(instruction.data)
  };
}
function getCloseExpiredPoolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([108, 212, 233, 53, 132, 83, 63, 219])
    })
  );
}
function getCloseExpiredPoolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getCloseExpiredPoolInstructionDataCodec() {
  return combineCodec(
    getCloseExpiredPoolInstructionDataEncoder(),
    getCloseExpiredPoolInstructionDataDecoder()
  );
}
function getCloseExpiredPoolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCloseExpiredPoolInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseExpiredPoolInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      rentPayer: getNextAccount(),
      owner: getNextAccount(),
      pool: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCloseExpiredPoolInstructionDataDecoder().decode(instruction.data)
  };
}
function getClosePoolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([140, 189, 209, 23, 239, 62, 239, 11])
    })
  );
}
function getClosePoolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getClosePoolInstructionDataCodec() {
  return combineCodec(
    getClosePoolInstructionDataEncoder(),
    getClosePoolInstructionDataDecoder()
  );
}
function getClosePoolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectTransactionSigner(
      accounts.owner.value
    ).address;
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getClosePoolInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseClosePoolInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      rentPayer: getNextAccount(),
      owner: getNextAccount(),
      pool: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getClosePoolInstructionDataDecoder().decode(instruction.data)
  };
}
function getCreatePoolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["poolId", fixEncoderSize(getBytesEncoder(), 32)],
      ["config", getPoolConfigEncoder()],
      ["currency", getOptionEncoder(getAddressEncoder())],
      ["sharedEscrow", getOptionEncoder(getAddressEncoder())],
      ["cosigner", getOptionEncoder(getAddressEncoder())],
      ["makerBroker", getOptionEncoder(getAddressEncoder())],
      ["orderType", getU8Encoder()],
      ["maxTakerSellCount", getOptionEncoder(getU32Encoder())],
      ["expireInSec", getOptionEncoder(getU64Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([233, 146, 209, 142, 207, 104, 64, 188]),
      currency: value.currency ?? none(),
      sharedEscrow: value.sharedEscrow ?? none(),
      cosigner: value.cosigner ?? none(),
      makerBroker: value.makerBroker ?? none(),
      orderType: value.orderType ?? 0,
      maxTakerSellCount: value.maxTakerSellCount ?? none(),
      expireInSec: value.expireInSec ?? none()
    })
  );
}
function getCreatePoolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["poolId", fixDecoderSize(getBytesDecoder(), 32)],
    ["config", getPoolConfigDecoder()],
    ["currency", getOptionDecoder(getAddressDecoder())],
    ["sharedEscrow", getOptionDecoder(getAddressDecoder())],
    ["cosigner", getOptionDecoder(getAddressDecoder())],
    ["makerBroker", getOptionDecoder(getAddressDecoder())],
    ["orderType", getU8Decoder()],
    ["maxTakerSellCount", getOptionDecoder(getU32Decoder())],
    ["expireInSec", getOptionDecoder(getU64Decoder())]
  ]);
}
function getCreatePoolInstructionDataCodec() {
  return combineCodec(
    getCreatePoolInstructionDataEncoder(),
    getCreatePoolInstructionDataDecoder()
  );
}
async function getCreatePoolInstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!args.poolId) {
    args.poolId = resolvePoolIdOnCreate();
  }
  if (!accounts.pool.value) {
    accounts.pool.value = await findPoolPda({
      owner: expectAddress(accounts.owner.value),
      poolId: expectSome(args.poolId)
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCreatePoolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getCreatePoolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: false },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!args.poolId) {
    args.poolId = resolvePoolIdOnCreate();
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCreatePoolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCreatePoolInstruction(instruction) {
  if (instruction.accounts.length < 5) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      rentPayer: getNextAccount(),
      owner: getNextAccount(),
      pool: getNextAccount(),
      whitelist: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCreatePoolInstructionDataDecoder().decode(instruction.data)
  };
}
function getDepositNftInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([93, 226, 132, 166, 141, 9, 48, 101]),
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getDepositNftInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getDepositNftInstructionDataCodec() {
  return combineCodec(
    getDepositNftInstructionDataEncoder(),
    getDepositNftInstructionDataDecoder()
  );
}
async function getDepositNftInstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ownerTokenRecord.value) {
    accounts.ownerTokenRecord = {
      ...accounts.ownerTokenRecord,
      ...await resolveOwnerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.poolTokenRecord.value) {
    accounts.poolTokenRecord = {
      ...accounts.poolTokenRecord,
      ...await resolvePoolTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram)
    ],
    programAddress,
    data: getDepositNftInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getDepositNftInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram)
    ],
    programAddress,
    data: getDepositNftInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDepositNftInstruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      pool: getNextAccount(),
      whitelist: getNextAccount(),
      ownerTa: getNextAccount(),
      poolTa: getNextAccount(),
      mint: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      metadata: getNextAccount(),
      mintProof: getNextOptionalAccount(),
      edition: getNextAccount(),
      ownerTokenRecord: getNextOptionalAccount(),
      poolTokenRecord: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount()
    },
    data: getDepositNftInstructionDataDecoder().decode(instruction.data)
  };
}
function getDepositNftT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([208, 34, 6, 147, 95, 218, 49, 160])
    })
  );
}
function getDepositNftT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getDepositNftT22InstructionDataCodec() {
  return combineCodec(
    getDepositNftT22InstructionDataEncoder(),
    getDepositNftT22InstructionDataDecoder()
  );
}
async function getDepositNftT22InstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = args.transferHookAccounts.map(
    (address3) => ({ address: address3, role: AccountRole.READONLY })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getDepositNftT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function getDepositNftT22Instruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = args.transferHookAccounts.map(
    (address3) => ({ address: address3, role: AccountRole.READONLY })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getDepositNftT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseDepositNftT22Instruction(instruction) {
  if (instruction.accounts.length < 11) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      pool: getNextAccount(),
      whitelist: getNextAccount(),
      mintProof: getNextAccount(),
      mint: getNextAccount(),
      ownerTa: getNextAccount(),
      poolTa: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getDepositNftT22InstructionDataDecoder().decode(instruction.data)
  };
}
function getDepositSolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["lamports", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([108, 81, 78, 117, 125, 155, 56, 200])
    })
  );
}
function getDepositSolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["lamports", getU64Decoder()]
  ]);
}
function getDepositSolInstructionDataCodec() {
  return combineCodec(
    getDepositSolInstructionDataEncoder(),
    getDepositSolInstructionDataDecoder()
  );
}
function getDepositSolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDepositSolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDepositSolInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      pool: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getDepositSolInstructionDataDecoder().decode(instruction.data)
  };
}
function getEditPoolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["newConfig", getOptionEncoder(getPoolConfigEncoder())],
      ["cosigner", getOptionEncoder(getAddressEncoder())],
      ["expireInSec", getOptionEncoder(getU64Encoder())],
      ["maxTakerSellCount", getOptionEncoder(getU32Encoder())],
      ["resetPriceOffset", getBooleanEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([50, 174, 34, 36, 3, 166, 29, 204]),
      cosigner: value.cosigner ?? none(),
      expireInSec: value.expireInSec ?? none(),
      maxTakerSellCount: value.maxTakerSellCount ?? none()
    })
  );
}
function getEditPoolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["newConfig", getOptionDecoder(getPoolConfigDecoder())],
    ["cosigner", getOptionDecoder(getAddressDecoder())],
    ["expireInSec", getOptionDecoder(getU64Decoder())],
    ["maxTakerSellCount", getOptionDecoder(getU32Decoder())],
    ["resetPriceOffset", getBooleanDecoder()]
  ]);
}
function getEditPoolInstructionDataCodec() {
  return combineCodec(
    getEditPoolInstructionDataEncoder(),
    getEditPoolInstructionDataDecoder()
  );
}
function getEditPoolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: false },
    pool: { value: input.pool ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getEditPoolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseEditPoolInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      pool: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getEditPoolInstructionDataDecoder().decode(instruction.data)
  };
}
function getSellNftTokenPoolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minPrice", getU64Encoder()],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([57, 44, 192, 48, 83, 8, 107, 48]),
      authorizationData: value.authorizationData ?? none(),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? none()
    })
  );
}
function getSellNftTokenPoolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minPrice", getU64Decoder()],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())]
  ]);
}
function getSellNftTokenPoolInstructionDataCodec() {
  return combineCodec(
    getSellNftTokenPoolInstructionDataEncoder(),
    getSellNftTokenPoolInstructionDataDecoder()
  );
}
async function getSellNftTokenPoolInstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    sellerTokenRecord: {
      value: input.sellerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromPool(resolverScope)
    };
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.sellerTa.value) {
    accounts.sellerTa = {
      ...accounts.sellerTa,
      ...await resolveSellerAta(resolverScope)
    };
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ownerTokenRecord.value) {
    accounts.ownerTokenRecord = {
      ...accounts.ownerTokenRecord,
      ...await resolveOwnerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sellerTokenRecord.value) {
    accounts.sellerTokenRecord = {
      ...accounts.sellerTokenRecord,
      ...await resolveSellerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.poolTokenRecord.value) {
    accounts.poolTokenRecord = {
      ...accounts.poolTokenRecord,
      ...await resolvePoolTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.sellerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTokenPoolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getSellNftTokenPoolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    sellerTokenRecord: {
      value: input.sellerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.sellerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTokenPoolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseSellNftTokenPoolInstruction(instruction) {
  if (instruction.accounts.length < 29) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      seller: getNextAccount(),
      rentPayer: getNextAccount(),
      feeVault: getNextAccount(),
      pool: getNextAccount(),
      whitelist: getNextAccount(),
      mintProof: getNextOptionalAccount(),
      sellerTa: getNextAccount(),
      ownerTa: getNextAccount(),
      poolTa: getNextAccount(),
      mint: getNextAccount(),
      metadata: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      edition: getNextAccount(),
      ownerTokenRecord: getNextOptionalAccount(),
      sellerTokenRecord: getNextOptionalAccount(),
      poolTokenRecord: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      sharedEscrow: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      takerBroker: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount(),
      ammProgram: getNextAccount(),
      escrowProgram: getNextOptionalAccount()
    },
    data: getSellNftTokenPoolInstructionDataDecoder().decode(instruction.data)
  };
}
function getSellNftTokenPoolT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minPrice", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([149, 234, 31, 103, 26, 36, 166, 49])
    })
  );
}
function getSellNftTokenPoolT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minPrice", getU64Decoder()]
  ]);
}
function getSellNftTokenPoolT22InstructionDataCodec() {
  return combineCodec(
    getSellNftTokenPoolT22InstructionDataEncoder(),
    getSellNftTokenPoolT22InstructionDataDecoder()
  );
}
async function getSellNftTokenPoolT22InstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromPool(resolverScope)
    };
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.sellerTa.value) {
    accounts.sellerTa = {
      ...accounts.sellerTa,
      ...await resolveSellerAta(resolverScope)
    };
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...args.transferHookAccounts.map((address3) => ({
      address: address3,
      role: AccountRole.READONLY
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTokenPoolT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getSellNftTokenPoolT22Instruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    rentPayer: { value: input.rentPayer ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.rentPayer.value) {
    accounts.rentPayer.value = expectSome(accounts.owner.value);
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...args.transferHookAccounts.map((address3) => ({
      address: address3,
      role: AccountRole.READONLY
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.rentPayer),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTokenPoolT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseSellNftTokenPoolT22Instruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      seller: getNextAccount(),
      rentPayer: getNextAccount(),
      feeVault: getNextAccount(),
      pool: getNextAccount(),
      whitelist: getNextAccount(),
      mintProof: getNextOptionalAccount(),
      sellerTa: getNextAccount(),
      ownerTa: getNextAccount(),
      mint: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      sharedEscrow: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      takerBroker: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount(),
      ammProgram: getNextAccount(),
      escrowProgram: getNextOptionalAccount()
    },
    data: getSellNftTokenPoolT22InstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getSellNftTradePoolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minPrice", getU64Encoder()],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ],
      ["optionalRoyaltyPct", getOptionEncoder(getU16Encoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([131, 82, 125, 77, 13, 157, 36, 90]),
      authorizationData: value.authorizationData ?? none(),
      optionalRoyaltyPct: value.optionalRoyaltyPct ?? none()
    })
  );
}
function getSellNftTradePoolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minPrice", getU64Decoder()],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())],
    ["optionalRoyaltyPct", getOptionDecoder(getU16Decoder())]
  ]);
}
function getSellNftTradePoolInstructionDataCodec() {
  return combineCodec(
    getSellNftTradePoolInstructionDataEncoder(),
    getSellNftTradePoolInstructionDataDecoder()
  );
}
async function getSellNftTradePoolInstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    mint: { value: input.mint ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    metadata: { value: input.metadata ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    sellerTokenRecord: {
      value: input.sellerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromPool(resolverScope)
    };
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.sellerTa.value) {
    accounts.sellerTa = {
      ...accounts.sellerTa,
      ...await resolveSellerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sellerTokenRecord.value) {
    accounts.sellerTokenRecord = {
      ...accounts.sellerTokenRecord,
      ...await resolveSellerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.poolTokenRecord.value) {
    accounts.poolTokenRecord = {
      ...accounts.poolTokenRecord,
      ...await resolvePoolTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.sellerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTradePoolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getSellNftTradePoolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    mint: { value: input.mint ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    metadata: { value: input.metadata ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    edition: { value: input.edition ?? null, isWritable: false },
    sellerTokenRecord: {
      value: input.sellerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = (args.creators ?? []).map(
    (address3) => ({ address: address3, role: AccountRole.WRITABLE })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.sellerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRulesProgram),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTradePoolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseSellNftTradePoolInstruction(instruction) {
  if (instruction.accounts.length < 27) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      seller: getNextAccount(),
      feeVault: getNextAccount(),
      pool: getNextAccount(),
      whitelist: getNextAccount(),
      mintProof: getNextOptionalAccount(),
      mint: getNextAccount(),
      sellerTa: getNextAccount(),
      poolTa: getNextAccount(),
      metadata: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      edition: getNextAccount(),
      sellerTokenRecord: getNextOptionalAccount(),
      poolTokenRecord: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      sharedEscrow: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      takerBroker: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount(),
      ammProgram: getNextAccount(),
      escrowProgram: getNextOptionalAccount()
    },
    data: getSellNftTradePoolInstructionDataDecoder().decode(instruction.data)
  };
}
function getSellNftTradePoolT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["minPrice", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([124, 145, 23, 52, 72, 113, 85, 9])
    })
  );
}
function getSellNftTradePoolT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["minPrice", getU64Decoder()]
  ]);
}
function getSellNftTradePoolT22InstructionDataCodec() {
  return combineCodec(
    getSellNftTradePoolT22InstructionDataEncoder(),
    getSellNftTradePoolT22InstructionDataDecoder()
  );
}
async function getSellNftTradePoolT22InstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    mint: { value: input.mint ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.feeVault.value) {
    accounts.feeVault = {
      ...accounts.feeVault,
      ...await resolveFeeVaultPdaFromPool(resolverScope)
    };
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.sellerTa.value) {
    accounts.sellerTa = {
      ...accounts.sellerTa,
      ...await resolveSellerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...args.transferHookAccounts.map((address3) => ({
      address: address3,
      role: AccountRole.READONLY
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTradePoolT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getSellNftTradePoolT22Instruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    seller: { value: input.seller ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    whitelist: { value: input.whitelist ?? null, isWritable: false },
    mintProof: { value: input.mintProof ?? null, isWritable: false },
    mint: { value: input.mint ?? null, isWritable: false },
    sellerTa: { value: input.sellerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    sharedEscrow: { value: input.sharedEscrow ?? null, isWritable: true },
    makerBroker: { value: input.makerBroker ?? null, isWritable: true },
    takerBroker: { value: input.takerBroker ?? null, isWritable: true },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    ammProgram: { value: input.ammProgram ?? null, isWritable: false },
    escrowProgram: { value: input.escrowProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.ammProgram.value) {
    accounts.ammProgram.value = "TAMM6ub33ij1mbetoMyVBLeKY5iP41i4UPUJQGkhfsg";
  }
  const remainingAccounts = [
    ...(args.creators ?? []).map((address3) => ({
      address: address3,
      role: AccountRole.WRITABLE
    })),
    ...args.transferHookAccounts.map((address3) => ({
      address: address3,
      role: AccountRole.READONLY
    }))
  ];
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.seller),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.whitelist),
      getAccountMeta(accounts.mintProof),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.sellerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.sharedEscrow),
      getAccountMeta(accounts.makerBroker),
      getAccountMeta(accounts.takerBroker),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.ammProgram),
      getAccountMeta(accounts.escrowProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getSellNftTradePoolT22InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseSellNftTradePoolT22Instruction(instruction) {
  if (instruction.accounts.length < 19) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      seller: getNextAccount(),
      feeVault: getNextAccount(),
      pool: getNextAccount(),
      whitelist: getNextAccount(),
      mintProof: getNextOptionalAccount(),
      mint: getNextAccount(),
      sellerTa: getNextAccount(),
      poolTa: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      sharedEscrow: getNextOptionalAccount(),
      makerBroker: getNextOptionalAccount(),
      takerBroker: getNextOptionalAccount(),
      cosigner: getNextOptionalAccount(),
      ammProgram: getNextAccount(),
      escrowProgram: getNextOptionalAccount()
    },
    data: getSellNftTradePoolT22InstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getTammNoopInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["event", getTAmmEventEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([31, 162, 228, 158, 153, 160, 198, 182])
    })
  );
}
function getTammNoopInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["event", getTAmmEventDecoder()]
  ]);
}
function getTammNoopInstructionDataCodec() {
  return combineCodec(
    getTammNoopInstructionDataEncoder(),
    getTammNoopInstructionDataDecoder()
  );
}
function getTammNoopInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    pool: { value: input.pool ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [getAccountMeta(accounts.pool)],
    programAddress,
    data: getTammNoopInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTammNoopInstruction(instruction) {
  if (instruction.accounts.length < 1) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      pool: getNextAccount()
    },
    data: getTammNoopInstructionDataDecoder().decode(instruction.data)
  };
}
function getWithdrawNftInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      [
        "authorizationData",
        getOptionEncoder(getAuthorizationDataLocalEncoder())
      ]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([142, 181, 191, 149, 82, 175, 216, 100]),
      authorizationData: value.authorizationData ?? none()
    })
  );
}
function getWithdrawNftInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["authorizationData", getOptionDecoder(getAuthorizationDataLocalDecoder())]
  ]);
}
function getWithdrawNftInstructionDataCodec() {
  return combineCodec(
    getWithdrawNftInstructionDataEncoder(),
    getWithdrawNftInstructionDataDecoder()
  );
}
async function getWithdrawNftInstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.metadata.value) {
    accounts.metadata = {
      ...accounts.metadata,
      ...await resolveMetadata(resolverScope)
    };
  }
  if (!accounts.edition.value) {
    accounts.edition = {
      ...accounts.edition,
      ...await resolveEditionFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.ownerTokenRecord.value) {
    accounts.ownerTokenRecord = {
      ...accounts.ownerTokenRecord,
      ...await resolveOwnerTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.poolTokenRecord.value) {
    accounts.poolTokenRecord = {
      ...accounts.poolTokenRecord,
      ...await resolvePoolTokenRecordFromTokenStandard(resolverScope)
    };
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram)
    ],
    programAddress,
    data: getWithdrawNftInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getWithdrawNftInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    metadata: { value: input.metadata ?? null, isWritable: true },
    edition: { value: input.edition ?? null, isWritable: false },
    ownerTokenRecord: {
      value: input.ownerTokenRecord ?? null,
      isWritable: true
    },
    poolTokenRecord: { value: input.poolTokenRecord ?? null, isWritable: true },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    sysvarInstructions: {
      value: input.sysvarInstructions ?? null,
      isWritable: false
    },
    authorizationRules: {
      value: input.authorizationRules ?? null,
      isWritable: false
    },
    authorizationRulesProgram: {
      value: input.authorizationRulesProgram ?? null,
      isWritable: false
    }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!args.tokenStandard) {
    args.tokenStandard = TokenStandard.ProgrammableNonFungible;
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram = {
      ...accounts.tokenMetadataProgram,
      ...resolveTokenMetadataProgramFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.sysvarInstructions.value) {
    accounts.sysvarInstructions = {
      ...accounts.sysvarInstructions,
      ...resolveSysvarInstructionsFromTokenStandard(resolverScope)
    };
  }
  if (!accounts.authorizationRulesProgram.value) {
    accounts.authorizationRulesProgram = {
      ...accounts.authorizationRulesProgram,
      ...resolveAuthorizationRulesProgramFromTokenStandard(resolverScope)
    };
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.edition),
      getAccountMeta(accounts.ownerTokenRecord),
      getAccountMeta(accounts.poolTokenRecord),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.sysvarInstructions),
      getAccountMeta(accounts.authorizationRules),
      getAccountMeta(accounts.authorizationRulesProgram)
    ],
    programAddress,
    data: getWithdrawNftInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseWithdrawNftInstruction(instruction) {
  if (instruction.accounts.length < 17) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === TENSOR_AMM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      pool: getNextAccount(),
      mint: getNextAccount(),
      ownerTa: getNextAccount(),
      poolTa: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount(),
      metadata: getNextAccount(),
      edition: getNextAccount(),
      ownerTokenRecord: getNextOptionalAccount(),
      poolTokenRecord: getNextOptionalAccount(),
      tokenMetadataProgram: getNextOptionalAccount(),
      sysvarInstructions: getNextOptionalAccount(),
      authorizationRules: getNextOptionalAccount(),
      authorizationRulesProgram: getNextOptionalAccount()
    },
    data: getWithdrawNftInstructionDataDecoder().decode(instruction.data)
  };
}
function getWithdrawNftT22InstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([112, 55, 80, 231, 181, 190, 92, 12])
    })
  );
}
function getWithdrawNftT22InstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getWithdrawNftT22InstructionDataCodec() {
  return combineCodec(
    getWithdrawNftT22InstructionDataEncoder(),
    getWithdrawNftT22InstructionDataDecoder()
  );
}
async function getWithdrawNftT22InstructionAsync(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const resolverScope = { programAddress, accounts, args };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.ownerTa.value) {
    accounts.ownerTa = {
      ...accounts.ownerTa,
      ...await resolveOwnerAta(resolverScope)
    };
  }
  if (!accounts.poolTa.value) {
    accounts.poolTa = {
      ...accounts.poolTa,
      ...await resolvePoolAta(resolverScope)
    };
  }
  if (!accounts.nftReceipt.value) {
    accounts.nftReceipt = {
      ...accounts.nftReceipt,
      ...await resolvePoolNftReceipt(resolverScope)
    };
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = args.transferHookAccounts.map(
    (address3) => ({ address: address3, role: AccountRole.READONLY })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getWithdrawNftT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function getWithdrawNftT22Instruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: false },
    ownerTa: { value: input.ownerTa ?? null, isWritable: true },
    poolTa: { value: input.poolTa ?? null, isWritable: true },
    nftReceipt: { value: input.nftReceipt ?? null, isWritable: true },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb";
  }
  if (!accounts.associatedTokenProgram.value) {
    accounts.associatedTokenProgram.value = "ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const remainingAccounts = args.transferHookAccounts.map(
    (address3) => ({ address: address3, role: AccountRole.READONLY })
  );
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.ownerTa),
      getAccountMeta(accounts.poolTa),
      getAccountMeta(accounts.nftReceipt),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.systemProgram),
      ...remainingAccounts
    ],
    programAddress,
    data: getWithdrawNftT22InstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseWithdrawNftT22Instruction(instruction) {
  if (instruction.accounts.length < 9) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      pool: getNextAccount(),
      mint: getNextAccount(),
      ownerTa: getNextAccount(),
      poolTa: getNextAccount(),
      nftReceipt: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getWithdrawNftT22InstructionDataDecoder().decode(instruction.data)
  };
}
function getWithdrawSolInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["lamports", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([145, 131, 74, 136, 65, 137, 42, 38])
    })
  );
}
function getWithdrawSolInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["lamports", getU64Decoder()]
  ]);
}
function getWithdrawSolInstructionDataCodec() {
  return combineCodec(
    getWithdrawSolInstructionDataEncoder(),
    getWithdrawSolInstructionDataDecoder()
  );
}
function getWithdrawSolInstruction(input) {
  const programAddress = TENSOR_AMM_PROGRAM_ADDRESS;
  const originalAccounts = {
    owner: { value: input.owner ?? null, isWritable: true },
    pool: { value: input.pool ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.pool),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getWithdrawSolInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseWithdrawSolInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      owner: getNextAccount(),
      pool: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getWithdrawSolInstructionDataDecoder().decode(instruction.data)
  };
}

export { CurveType, DEFAULT_ADDRESS, Direction, PoolType, TENSOR_AMM_ERROR__ARITHMETIC_ERROR, TENSOR_AMM_ERROR__BAD_COSIGNER, TENSOR_AMM_ERROR__BAD_ESCROW_ACCOUNT, TENSOR_AMM_ERROR__BAD_FEE_ACCOUNT, TENSOR_AMM_ERROR__BAD_METADATA, TENSOR_AMM_ERROR__BAD_MINT_PROOF, TENSOR_AMM_ERROR__BAD_OWNER, TENSOR_AMM_ERROR__BAD_ROYALTIES_PCT, TENSOR_AMM_ERROR__BAD_RULE_SET, TENSOR_AMM_ERROR__BAD_SHARED_ESCROW, TENSOR_AMM_ERROR__BAD_WHITELIST, TENSOR_AMM_ERROR__CREATOR_MISMATCH, TENSOR_AMM_ERROR__DELTA_TOO_LARGE, TENSOR_AMM_ERROR__ESCROW_PROGRAM_NOT_SET, TENSOR_AMM_ERROR__EXISTING_NFTS, TENSOR_AMM_ERROR__EXPIRED_POOL, TENSOR_AMM_ERROR__EXPIRY_TOO_LARGE, TENSOR_AMM_ERROR__FEES_NOT_ALLOWED, TENSOR_AMM_ERROR__FEES_TOO_HIGH, TENSOR_AMM_ERROR__FROZEN_AMOUNT_MISMATCH, TENSOR_AMM_ERROR__INSUFFICIENT_TSWAP_ACC_BALANCE, TENSOR_AMM_ERROR__INVALID_POOL_AMOUNT, TENSOR_AMM_ERROR__INVALID_PROOF, TENSOR_AMM_ERROR__MAX_TAKER_SELL_COUNT_EXCEEDED, TENSOR_AMM_ERROR__MAX_TAKER_SELL_COUNT_TOO_SMALL, TENSOR_AMM_ERROR__MINT_PROOF_NOT_SET, TENSOR_AMM_ERROR__MISSING_COSIGNER, TENSOR_AMM_ERROR__MISSING_FEES, TENSOR_AMM_ERROR__MISSING_MAKER_BROKER, TENSOR_AMM_ERROR__POOLS_ARE_THE_SAME, TENSOR_AMM_ERROR__POOL_FEES_COMPOUNDED, TENSOR_AMM_ERROR__POOL_FROZEN, TENSOR_AMM_ERROR__POOL_KEEP_ALIVE, TENSOR_AMM_ERROR__POOL_NOT_EXPIRED, TENSOR_AMM_ERROR__POOL_NOT_ON_SHARED_ESCROW, TENSOR_AMM_ERROR__POOL_ON_SHARED_ESCROW, TENSOR_AMM_ERROR__PRICE_MISMATCH, TENSOR_AMM_ERROR__ROYALTIES_ENABLED, TENSOR_AMM_ERROR__SHARED_ESCROW_IN_USE, TENSOR_AMM_ERROR__SPL_TOKENS_NOT_SUPPORTED, TENSOR_AMM_ERROR__STARTING_PRICE_TOO_SMALL, TENSOR_AMM_ERROR__UNSUPPORTED_CURRENCY, TENSOR_AMM_ERROR__WHITELIST_NOT_VERIFIED, TENSOR_AMM_ERROR__WRONG_AUTHORITY, TENSOR_AMM_ERROR__WRONG_FROZEN_STATUS, TENSOR_AMM_ERROR__WRONG_MAKER_BROKER, TENSOR_AMM_ERROR__WRONG_MINT, TENSOR_AMM_ERROR__WRONG_ORDER_TYPE, TENSOR_AMM_ERROR__WRONG_OWNER, TENSOR_AMM_ERROR__WRONG_POOL, TENSOR_AMM_ERROR__WRONG_POOL_TYPE, TENSOR_AMM_ERROR__WRONG_POOL_VERSION, TENSOR_AMM_ERROR__WRONG_RENT_PAYER, TENSOR_AMM_PROGRAM_ADDRESS, TakerSide, TensorAmmAccount, TensorAmmInstruction, calculatePrice, decodeNftDepositReceipt, decodePool, fetchAllMaybeNftDepositReceipt, fetchAllMaybePool, fetchAllNftDepositReceipt, fetchAllPool, fetchMaybeNftDepositReceipt, fetchMaybeNftDepositReceiptFromSeeds, fetchMaybePool, fetchMaybePoolFromSeeds, fetchNftDepositReceipt, fetchNftDepositReceiptFromSeeds, fetchPool, fetchPoolFromSeeds, findNftDepositReceiptPda, findPoolPda, getAmountOfBids, getAuthorizationDataLocalCodec, getAuthorizationDataLocalDecoder, getAuthorizationDataLocalEncoder, getBuyNftInstruction, getBuyNftInstructionAsync, getBuyNftInstructionDataCodec, getBuyNftInstructionDataDecoder, getBuyNftInstructionDataEncoder, getBuyNftT22Instruction, getBuyNftT22InstructionAsync, getBuyNftT22InstructionDataCodec, getBuyNftT22InstructionDataDecoder, getBuyNftT22InstructionDataEncoder, getBuySellEventCodec, getBuySellEventDecoder, getBuySellEventEncoder, getCloseExpiredPoolInstruction, getCloseExpiredPoolInstructionDataCodec, getCloseExpiredPoolInstructionDataDecoder, getCloseExpiredPoolInstructionDataEncoder, getClosePoolInstruction, getClosePoolInstructionDataCodec, getClosePoolInstructionDataDecoder, getClosePoolInstructionDataEncoder, getCreatePoolInstruction, getCreatePoolInstructionAsync, getCreatePoolInstructionDataCodec, getCreatePoolInstructionDataDecoder, getCreatePoolInstructionDataEncoder, getCurrencyCodec, getCurrencyDecoder, getCurrencyEncoder, getCurrentAskPrice, getCurrentBidPrice, getCurrentBidPriceSync, getCurveTypeCodec, getCurveTypeDecoder, getCurveTypeEncoder, getDepositNftInstruction, getDepositNftInstructionAsync, getDepositNftInstructionDataCodec, getDepositNftInstructionDataDecoder, getDepositNftInstructionDataEncoder, getDepositNftT22Instruction, getDepositNftT22InstructionAsync, getDepositNftT22InstructionDataCodec, getDepositNftT22InstructionDataDecoder, getDepositNftT22InstructionDataEncoder, getDepositSolInstruction, getDepositSolInstructionDataCodec, getDepositSolInstructionDataDecoder, getDepositSolInstructionDataEncoder, getDirectionCodec, getDirectionDecoder, getDirectionEncoder, getEditPoolInstruction, getEditPoolInstructionDataCodec, getEditPoolInstructionDataDecoder, getEditPoolInstructionDataEncoder, getNeededBalanceForBidQuantity, getNftDepositReceiptCodec, getNftDepositReceiptDecoder, getNftDepositReceiptEncoder, getNftDepositReceiptSize, getNullableAddressCodec, getNullableAddressDecoder, getNullableAddressEncoder, getNullableU16Codec, getNullableU16Decoder, getNullableU16Encoder, getPayloadTypeLocalCodec, getPayloadTypeLocalDecoder, getPayloadTypeLocalEncoder, getPoolCodec, getPoolConfigCodec, getPoolConfigDecoder, getPoolConfigEncoder, getPoolDecoder, getPoolEncoder, getPoolSize, getPoolStatsCodec, getPoolStatsDecoder, getPoolStatsEncoder, getPoolTypeCodec, getPoolTypeDecoder, getPoolTypeEncoder, getProofInfoLocalCodec, getProofInfoLocalDecoder, getProofInfoLocalEncoder, getSeedsVecLocalCodec, getSeedsVecLocalDecoder, getSeedsVecLocalEncoder, getSellNftTokenPoolInstruction, getSellNftTokenPoolInstructionAsync, getSellNftTokenPoolInstructionDataCodec, getSellNftTokenPoolInstructionDataDecoder, getSellNftTokenPoolInstructionDataEncoder, getSellNftTokenPoolT22Instruction, getSellNftTokenPoolT22InstructionAsync, getSellNftTokenPoolT22InstructionDataCodec, getSellNftTokenPoolT22InstructionDataDecoder, getSellNftTokenPoolT22InstructionDataEncoder, getSellNftTradePoolInstruction, getSellNftTradePoolInstructionAsync, getSellNftTradePoolInstructionDataCodec, getSellNftTradePoolInstructionDataDecoder, getSellNftTradePoolInstructionDataEncoder, getSellNftTradePoolT22Instruction, getSellNftTradePoolT22InstructionAsync, getSellNftTradePoolT22InstructionDataCodec, getSellNftTradePoolT22InstructionDataDecoder, getSellNftTradePoolT22InstructionDataEncoder, getTAmmEventCodec, getTAmmEventDecoder, getTAmmEventEncoder, getTaggedPayloadCodec, getTaggedPayloadDecoder, getTaggedPayloadEncoder, getTakerSideCodec, getTakerSideDecoder, getTakerSideEncoder, getTammNoopInstruction, getTammNoopInstructionDataCodec, getTammNoopInstructionDataDecoder, getTammNoopInstructionDataEncoder, getTensorAmmErrorMessage, getWithdrawNftInstruction, getWithdrawNftInstructionAsync, getWithdrawNftInstructionDataCodec, getWithdrawNftInstructionDataDecoder, getWithdrawNftInstructionDataEncoder, getWithdrawNftT22Instruction, getWithdrawNftT22InstructionAsync, getWithdrawNftT22InstructionDataCodec, getWithdrawNftT22InstructionDataDecoder, getWithdrawNftT22InstructionDataEncoder, getWithdrawSolInstruction, getWithdrawSolInstructionDataCodec, getWithdrawSolInstructionDataDecoder, getWithdrawSolInstructionDataEncoder, identifyTensorAmmAccount, identifyTensorAmmInstruction, isPayloadTypeLocal, isSol, isTAmmEvent, parseBuyNftInstruction, parseBuyNftT22Instruction, parseCloseExpiredPoolInstruction, parseClosePoolInstruction, parseCreatePoolInstruction, parseDepositNftInstruction, parseDepositNftT22Instruction, parseDepositSolInstruction, parseEditPoolInstruction, parseSellNftTokenPoolInstruction, parseSellNftTokenPoolT22Instruction, parseSellNftTradePoolInstruction, parseSellNftTradePoolT22Instruction, parseTammNoopInstruction, parseWithdrawNftInstruction, parseWithdrawNftT22Instruction, parseWithdrawSolInstruction, payloadTypeLocal, resolveFeeVaultPdaFromPool, resolvePoolIdOnCreate, solCurrency, tAmmEvent };
//# sourceMappingURL=index.mjs.map
//# sourceMappingURL=index.mjs.map